note
  description  : "System's root class."

class MAIN
  inherit ES_TEST

create  make

--------------------------------------------------------------------------------
feature -- Initialization

make
  do
    add_boolean_case (agent evaluator_with_constant_341)
    add_boolean_case (agent evaluator_with_plus)
    add_boolean_case (agent evaluator_with_plus_times_1)
    add_boolean_case (agent evaluator_with_plus_times_2)
    add_boolean_case (agent evaluator_with_minus)

    add_boolean_case (agent infix_printer_with_constant)
    add_boolean_case (agent infix_printer_with_plus)
    add_boolean_case (agent infix_printer_with_plus_times_1)
    add_boolean_case (agent infix_printer_with_plus_times_2)

    add_boolean_case (agent postfix_printer_with_constant)
    add_boolean_case (agent postfix_printer_with_plus)
    add_boolean_case (agent postfix_printer_with_plus_times_1)
    add_boolean_case (agent postfix_printer_with_plus_times_2)

    add_boolean_case (agent prefix_printer_with_constant)
    add_boolean_case (agent prefix_printer_with_plus)
    add_boolean_case (agent prefix_printer_with_plus_times_1)
    add_boolean_case (agent prefix_printer_with_plus_times_2)
    create_expressions
    show_browser
    run_espec
  end

--------------------------------------------------------------------------------
c2, c3, c5, c341 : CONSTANT
plus : ADDITION
times : MULTIPLICATION
expr_1, expr_2, expr_3, expr_4 : EXPRESSION

evaluator: EVALUATOR
infix_printer : INFIX_PRINTER
postfix_printer : POSTFIX_PRINTER
prefix_printer : PREFIX_PRINTER

create_expressions
  do
    create c2.make (2) ; create c3.make (3) ; create c5.make (5)
    create c341.make (341)

    expr_1 := c341

    create plus.make (c341, c2)
    expr_2 := plus

    create plus.make (c2, c3)
    create times.make(plus, c5)
    expr_3 := times

    create times.make(c3, c5)
    create plus.make (c2, times)
    expr_4 := plus
  end

--------------------------------------------------------------------------------
feature -- Test cases Visitor Evaluator

evaluator_with_constant_341: BOOLEAN
  do
    comment("Evaluator expr is constant 341")
  	create evaluator
    expr_1.accept (evaluator)
    Result := evaluator.value = c341.value
  end

evaluator_with_plus: BOOLEAN
  do
    comment("Evaluator expr is 341 + 2 ")
  	create evaluator
    expr_2.accept (evaluator)
    Result := evaluator.value = 343
  end

evaluator_with_plus_times_1: BOOLEAN
  do
    comment("Evaluator expr is (2 + 3) * 5")
  	create evaluator
    expr_3.accept (evaluator)
    Result := evaluator.value = 25
  end

evaluator_with_plus_times_2: BOOLEAN
  do
    comment("Evaluator expr is 2 + (3 * 5)")
  	create evaluator
    expr_4.accept (evaluator)
    Result := evaluator.value = 17
  end

evaluator_with_minus: BOOLEAN
  local
    sub : SUBTRACTION
    times1 : MULTIPLICATION
    add1 : ADDITION
  do
    comment("Evaluator expr is (5 * 2) - (341 + 3)")
    create c341.make (341)
    create times1.make(c5, c2)
    create add1.make(c341, c3)
    create sub.make(times1, add1)
    sub.accept (evaluator)
    Result := evaluator.value = -334
  end
--------------------------------------------------------------------------------
feature -- Test cases Visitor Infix Printer

infix_printer_with_constant: BOOLEAN
  do
    comment("Infix expr is constant 341")
  	create infix_printer.make
    expr_1.accept (infix_printer)
    Result := equal(infix_printer.string, "341")
  end

infix_printer_with_plus: BOOLEAN
  do
    comment("Infix expr is 341 + 2")
  	create infix_printer.make
    expr_2.accept (infix_printer)
    Result := equal(infix_printer.string, "(341 + 2)")
  end

infix_printer_with_plus_times_1: BOOLEAN
  do
    comment("Infix expr is (2 + 3) * 5")
  	create infix_printer.make
    expr_3.accept (infix_printer)
    Result := equal(infix_printer.string, "((2 + 3) * 5)")
  end

infix_printer_with_plus_times_2: BOOLEAN
  do
    comment("Infix expr is 2 + (3 * 5)")
  	create infix_printer.make
    expr_4.accept (infix_printer)
    Result := equal(infix_printer.string, "(2 + (3 * 5))")
  end

--------------------------------------------------------------------------------
feature -- Test cases Visitor Postfix Printer

postfix_printer_with_constant: BOOLEAN
  do
    comment("Postfix expr is constant 341")
  	create postfix_printer.make
    expr_1.accept (postfix_printer)
    Result := equal(postfix_printer.string, "341 ")
  end

postfix_printer_with_plus: BOOLEAN
  do
    comment("Postfix expr is 341 + 2")
  	create postfix_printer.make
    expr_2.accept (postfix_printer)
    Result := equal(postfix_printer.string, "341 2 + ")
  end

postfix_printer_with_plus_times_1: BOOLEAN
  do
    comment("Postfix expr is (2 + 3) * 5")
  	create postfix_printer.make
    expr_3.accept (postfix_printer)
    Result := equal(postfix_printer.string, "2 3 + 5 * ")
  end

postfix_printer_with_plus_times_2: BOOLEAN
  do
    comment("Postfix expr is 2 + (3 * 5)")
  	create postfix_printer.make
    expr_4.accept (postfix_printer)
    Result := equal(postfix_printer.string, "2 3 5 * + ")
  end

--------------------------------------------------------------------------------
feature -- Test cases Visitor prefix Printer

prefix_printer_with_constant: BOOLEAN
  do
    comment("Prefix expr is constant 341")
  	create prefix_printer.make
    expr_1.accept (prefix_printer)
    Result := equal(prefix_printer.string, "341")
  end

prefix_printer_with_plus: BOOLEAN
  do
    comment("Prefix expr is 341 + 2")
  	create prefix_printer.make
    expr_2.accept (prefix_printer)
    Result := equal(prefix_printer.string, "add (341, 2)")
  end

prefix_printer_with_plus_times_1: BOOLEAN
  do
    comment("Prefix expr is (2 + 3) * 5")
  	create prefix_printer.make
    expr_3.accept (prefix_printer)
    Result := equal(prefix_printer.string, "mul (add (2, 3), 5)")
  end

prefix_printer_with_plus_times_2: BOOLEAN
  do
    comment("Prefix expr is 2 + (3 * 5)")
  	create prefix_printer.make
    expr_4.accept (prefix_printer)
    Result := equal(prefix_printer.string, "add (2, mul (3, 5))")
  end

end
